@@ -7,7 +7,7 @@
 #define _DEFAULT_SOURCE 1
 #include <stdlib.h>
 #include <string.h>
-#if !defined(WIN32) && !defined(CYGWIN)
+#if !defined(WIN32)
 #define __USE_BSD 1  /**< Use BSD code */
 #include <unistd.h>
 #include <netdb.h>
@@ -66,7 +66,7 @@ int net_hostaddr(int af, struct sa *ip)
  */
 int net_default_source_addr_get(int af, struct sa *ip)
 {
-#if defined(WIN32) || defined(CYGWIN)
+#if defined(WIN32)
 	return net_hostaddr(af, ip);
 #else
 	char ifname[64] = "";
