@@ -4,6 +4,10 @@
  * Copyright (C) 2010 Creytiv.com
  */
 
+/* also defined by wincrypt.h */
+#ifdef WIN32
+#undef X509_NAME
+#endif
 
 /*
  * Mapping of feature macros
@@ -25,6 +29,13 @@
 #endif
 
 
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+typedef X509_NAME*(tls_get_certfield_h)(const X509 *);
+#else
+typedef X509_NAME*(tls_get_certfield_h)(X509 *);
+#endif
+
+
 struct tls {
 	SSL_CTX *ctx;
 	X509 *cert;
